import os
import unittest
from .base_test import BaseTest
from ..checks import check_python_shebang, check_file_is_executable


class TestPythonChecks(BaseTest, unittest.TestCase):
    def test_python_shebang_check(self):
        py_file = open("script.py", "w")
        py_file.close()
        self.assertFalse(check_python_shebang("script.py"))
        py_file = open("script.py", "w")
        py_file.write("#!/usr/bin/python3\n")
        py_file.close()
        self.assertTrue(check_python_shebang("script.py"))

    def test_file_executable_check(self):
        f = open("script.py", "w")
        f.close()
        os.chmod("script.py", 0o777)
        self.assertTrue(check_file_is_executable("script.py"))


if __name__ == "__main__":
    unittest.main()



import subprocess
from ..utils.error_logging import print_please_install_semistandard


def check_javascript_shebang(file_path):
    flag = True
    with open(file_path, "r") as f:
        first_line = f.readline().strip()
        if first_line not in ("#!/usr/bin/node", "#!/usr/bin/env node"):
            flag = False
    return flag


def semistandard_check(file_path):
    try:
        result = subprocess.run(["semistandard", file_path])
    except Exception:
        print_please_install_semistandard()
        return False
    return result == 0


def check_no_var(file_path):
    with open(file_path, "r") as f:
        content = f.read()
        lines = content.split("\n")
        for line in lines:
            if line.strip().startswith("var "):
                return False
        return True



import subprocess
from ..utils.error_logging import print_please_install_betty


def betty_check(file_path):
    try:
        result = subprocess.run(["betty", file_path])
    except Exception:
        print_please_install_betty()
        return False
    return result.returncode == 0



import sys
from .checks import *
from .utils.helpers import *
from .utils.error_logging import *


def main():
    if not check_file_present("README.md"):
        print_file_not_present("README.md")
        sys.exit(1)
    if not check_file_not_empty("README.md"):
        print_file_empty("README.md")
    for root, dirs, files in os.walk("."):
        # exclude virtual environment folders
        for dir in dirs:
            if is_python_virtual_env_folder(dir):
                dirs.remove(dir)
                break
        # exclude .git folder
        if ".git" in dirs:
            dirs.remove(".git")
        for file in files:
            file_path = os.path.join(root, file)
            if file_path.endswith(
                (".c", ".py", ".js", ".m", ".h", ".mjs", ".jsx", ".json")
            ):
                if not check_file_ends_with_new_line(file_path):
                    if not is_empty_init_py(file_path):
                        print_no_ending_new_line(file_path)
            # c and c header files
            if file.endswith((".c", ".h")):
                betty_check(file_path)
            # python checks
            if file_path.endswith(".py") and not is_empty_init_py(file_path):
                if not check_file_is_executable(file_path):
                    print_file_not_executable(file_path)
                if not is_empty_init_py(file_path) and not check_python_shebang(
                    file_path
                ):
                    print_no_shebang(file_path)
                check_module_function_class_documentation(file_path)
                pycodestyle_check(file_path)
            # javascript checks
            if file.endswith(".js"):
                if is_nodejs_project():
                    if not check_file_is_executable(file_path):
                        print_file_not_executable(file_path)
                    if not check_javascript_shebang(file_path):
                        print_no_shebang(file_path)
                if not check_no_var(file_path):
                    print_var_was_used(file_path)
                semistandard_check(file_path)


if __name__ == "__main__":
    main()



import os, shutil


class BaseTest:
    def setUp(self) -> None:
        super().setUp()
        # create folder to be tested upon and set cwd
        os.mkdir("alx-project")
        os.chdir("alx-project")

    def tearDown(self) -> None:
        super().tearDown()
        # delete folder and reset cwd
        os.chdir("..")
        shutil.rmtree("alx-project")



from colorama import Fore, Back


def print_file_not_present(file_path):
    print(Fore.RED + f"{file_path} file Not Found" + Fore.RESET)


def print_file_empty(file_path):
    print(Fore.RED + f"{file_path} file is Empty" + Fore.RESET)


def print_no_ending_new_line(file_path):
    print(Fore.RED + f"No new line at end of file: {file_path}" + Fore.RESET)


def print_file_not_executable(file_path):
    print(Fore.RED + f"{file_path} is not Executable" + Fore.RESET)


def print_no_shebang(file_path):
    print(Fore.RED + f"No shebang in {file_path}" + Fore.RESET)


def print_no_module_docstring(file_path):
    print(Fore.RED + f"{file_path} does not have Module DocString" + Fore.RESET)


def print_no_function_docstring(file_path, function_name):
    print(
        Fore.RED
        + f"In {file_path}, the {function_name} function has no Function DocString"
        + Fore.RESET
    )


def print_no_class_docstring(file_path, class_name):
    print(
        Fore.RED
        + f"In {file_path}, the {class_name} class has no Class DocString"
        + Fore.RESET
    )


def print_please_install_semistandard():
    print(
        Fore.RED
        + "Please Install Semistandard with the following command:\n\t"
        + Fore.RESET
        + Back.RED
        + "npm i semistandard -g"
        + Back.RESET
    )


def print_please_install_betty():
    print(
        Fore.RED
        + "Please make sure "
        + Fore.RESET
        + Back.RED
        + " Betty "
        + Back.RESET
        + Fore.RED
        + " is Installed"
        + Fore.RESET
    )


def print_var_was_used(file_path):
    print(Fore.RED + f"In {file_path}, 'var' was used." + Fore.RESET)


def print_check_docstrings(file_path):
    print(Fore.RED + f"Error: Check docstrings in {file_path}" + Fore.RESET)



import unittest
import os
import subprocess
from .base_test import BaseTest
from ..utils.helpers import is_python_virtual_env_folder


class TestIsPythonVirtualEnvFolder(BaseTest, unittest.TestCase):
    def test_virtual_env_folder_exists(self):
        # Create a tmp virtual environment
        subprocess.run(["python", "-m", "venv", "venv"], stdout=subprocess.PIPE)

        # Test the function with the created folder
        venv_path = os.path.join(os.getcwd(), "venv")
        result = is_python_virtual_env_folder(venv_path)

        # Assert that the result is True, as the virtual environment folder exists
        self.assertTrue(result)

    def test_virtual_env_folder_does_not_exist(self):
        # Create a temporary directory without a virtual environment file
        folder_path = "temp_folder"
        os.makedirs(folder_path, exist_ok=True)

        # Test the function with the created folder
        result = is_python_virtual_env_folder(folder_path)

        # Assert that the result is False, as the virtual environment folder does not exist
        self.assertFalse(result)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from .base_test import BaseTest
from ..checks import check_javascript_shebang, check_file_is_executable, check_no_var


class TestJavascriptChecks(BaseTest, unittest.TestCase):
    def test_javascript_shebang_check(self):
        js_file = open("script.js", "w")
        js_file.close()
        self.assertFalse(check_javascript_shebang("script.js"))
        js_file = open("script.js", "w")
        js_file.write("#!/usr/bin/node\n")
        js_file.close()
        self.assertTrue(check_javascript_shebang("script.js"))

    def test_file_executable_check(self):
        f = open("script.js", "w")
        f.close()
        os.chmod("script.js", 0o777)
        self.assertTrue(check_file_is_executable("script.js"))

    def test_no_var(self):
        f = open("script.js", "w")
        f.write("var a = 10;")
        f.close()
        self.assertFalse(check_no_var("script.js"))
        f = open("script.js", "w")
        f.write("let a = 10;")
        f.close()
        self.assertTrue(check_no_var("script.js"))


if __name__ == "__main__":
    unittest.main()



import os
import ast
import subprocess
from ..utils.error_logging import (
    print_no_module_docstring,
    print_no_function_docstring,
    print_no_class_docstring,
    print_check_docstrings,
)


def check_file_is_executable(file_path):
    flag = True
    if not os.access(file_path, os.X_OK):
        flag = False
    return flag


def check_python_shebang(file_path):
    flag = True
    with open(file_path, "rb") as f:
        first_line = f.readline().strip()
        if first_line not in (b"#!/usr/bin/python3", b"#!/usr/bin/env python3"):
            flag = False
    return flag


def check_module_function_class_documentation(file_path):
    flag = True
    with open(file_path, "rb") as f:
        content = f.read()
        # remove shebang
        if content.startswith(b"#!"):
            if len(content.split(b"\n")) < 2:
                content = ""
            else:
                content = content.split(b"\n", 1)[1]
        tree = ast.parse(content)
        try:
            for node in ast.walk(tree):
                # check module docstring
                if isinstance(node, ast.Module):
                    if not isinstance(node.body[0].value, ast.Str):
                        flag = False
                        print_no_module_docstring(file_path)
                        return
                # check function docstring
                if isinstance(node, ast.FunctionDef) and not isinstance(
                    node.body[0].value, ast.Str
                ):
                    flag = False
                    print_no_function_docstring(file_path, node.name)
                # check class docstring
                if isinstance(node, ast.ClassDef) and not isinstance(
                    node.body[0].value, ast.Str
                ):
                    flag = False
                    print_no_class_docstring(file_path, node.name)
        except Exception:
            print_check_docstrings(file_path)
            return False
    return flag


def pycodestyle_check(file_path):
    try:
        result = subprocess.run(["pycodestyle", file_path])
    except Exception:
        return False
    return result == 0



import os
import sys


def is_python_virtual_env_folder(folder_path):
    return os.path.exists(f"{folder_path}/pyvenv.cfg")


def is_nodejs_project():
    """Checks for node.js command line switches"""
    switches = ("-js", "--nodejs-checks")
    for switch in switches:
        if switch in sys.argv:
            return True
    return False


def is_empty_init_py(file_path):
    """Checks if file is empty __init__.py"""
    if file_path.endswith('__init__.py'):
        from ..checks.general import check_file_not_empty
        if not check_file_not_empty(file_path):
            return True
    return False



import unittest
from .base_test import BaseTest
from ..checks import (
    check_file_present,
    check_file_not_empty,
    check_file_ends_with_new_line,
)


class TestGeneralChecks(BaseTest, unittest.TestCase):
    def test_readme_present_check(self):
        self.assertFalse(check_file_present("README.md"))
        readme_fd = open("README.md", "w")
        readme_fd.close()
        self.assertTrue(check_file_present("README.md"))

    def test_readme_not_empty_check(self):
        readme_fd = open("README.md", "w+")
        readme_fd.close()
        self.assertFalse(check_file_not_empty("README.md"))
        readme_fd = open("README.md", "w+")
        readme_fd.write("Project description")
        readme_fd.close()
        self.assertTrue(check_file_not_empty("README.md"))

    def test_files_with_new_line_check(self):
        f = open("file", "w")
        f.close()
        self.assertFalse(check_file_ends_with_new_line("file"))
        f = open("file", "wb")
        f.write(b"line\n\n")
        f.close()
        self.assertTrue(check_file_ends_with_new_line("file"))


if __name__ == "__main__":
    unittest.main()



import os


def check_file_present(file_path):
    if not os.path.exists(file_path):
        return False
    return True


def check_file_not_empty(file_path):
    if not check_file_present(file_path):
        return False
    if os.stat(file_path).st_size == 0:
        return False
    return True


def check_file_ends_with_new_line(file_path):
    if not check_file_not_empty(file_path):
        return False
    with open(file_path, "r") as f:
        return f.read()[-1] == "\n"
